<?php

namespace DCL\Admin;

/**
 * Admin: metaboxes.
 *
 * Defines metabox with checkbox to restrict
 * access on posts and pages.
 *
 * @package    DCL\Admin
 * @author     antwerpes ag <opensource@antwerpes.com>
 */
class DCL_Metaboxes extends DCL_Admin {

	/**
	 * Register meta box.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_register_meta_boxes() {
		add_meta_box(
			'dcl_restrict_access',
			__( 'DocCheck Login', 'doccheck-login' ),
			[ $this, 'dcl_display_meta_box_restrict_access' ],
			[ 'post', 'page' ],
			'side'
		);
	}

	public function dcl_register_meta_boxes_custom_post_type() {
		$screens = get_post_types();
		foreach ( $screens as $screen ) {
			add_meta_box(
				'dcl_restrict_access',
				__( 'DocCheck Login', 'doccheck-login' ),
				[ $this, 'dcl_display_meta_box_restrict_access' ],
				$screen,
				'side'
			);
		}

	}


	/**
	 * Meta box display callback.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_display_meta_box_restrict_access() { ?>
        <label>
            <input type="checkbox"
                   id="dcl_restrict_access"
                   name="dcl_restrict_access"
				<?php checked( get_post_meta( get_the_ID(), 'dcl_restrict_access', true ), 'on', true ); ?> />
			<?php _e( 'Restrict access with login', 'doccheck-login' ); ?>
        </label>

		<?php wp_nonce_field( 'dcl_restrict_access', 'dcl_restrict_access_nonce' ); ?>
	<?php }

	/**
	 * Save meta box content.
	 *
	 * @param int $post_id Post ID
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_save_meta_boxes( $post_id ) {
		$options = [
			'dcl_restrict_access'
		];

		foreach ( $options as $option ) {
			if ( $this->dcl_user_can_save( $post_id, $option . '_nonce', $option ) ) {
				if ( isset( $_POST[ $option ] ) ) {
					update_post_meta( $post_id, $option, $_POST[ $option ] );
				} else {
					delete_post_meta( $post_id, $option );
				}
			}
		}
	}

	/**
	 * Verifies that the user who is currently logged in has permission to save the data
	 * from the meta box to the database.
	 *
	 * @param integer $post_id The current post being saved.
	 * @param string $nonce The number used once to identify the serialization value
	 * @param string $action The source of the action of the nonce being used
	 *
	 * @return  boolean True if the user can save the information
	 * @since   1.0.0
	 * @access  private
	 */
	private function dcl_user_can_save( $post_id, $nonce, $action ) {
		$is_autosave    = wp_is_post_autosave( $post_id );
		$is_revision    = wp_is_post_revision( $post_id );
		$is_valid_nonce = ( isset( $_POST[ $nonce ] ) && wp_verify_nonce( $_POST[ $nonce ], $action ) );

		return ! ( $is_autosave || $is_revision ) && $is_valid_nonce;
	}
}