<?php

namespace DCL\Admin;

/**
 * Admin: settings.
 *
 * Registers plugin settings page.
 *
 * @package    DCL\Admin
 * @author     antwerpes ag <opensource@antwerpes.com>
 */
class DCL_Settings extends DCL_Admin {

	/**
	 * Support link to get a login id.
	 *
	 * @var string
	 */
	var $dcl_support_login_link = 'http://flexikon.doccheck.com/de/DocCheck:DocCheck_Login_f%C3%BCr_Ihre_Website';

	/**
	 * Support link to cream.
	 *
	 * @var string
	 */
	var $dcl_support_cream_link = 'https://crm.doccheck.com/';

	/**
	 * Support email adresse to get client secret.
	 *
	 * @var string
	 */
	var $dcl_support_client_secret_email = 'industry@doccheck.com';

	/**
	 * Register settings page.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_register_settings_page() {
		add_options_page(
			__( 'DocCheck Login', 'doccheck-login' ),
			__( 'DocCheck Login', 'doccheck-login' ),
			'manage_options',
			$this->plugin_name,
			[ $this, 'dcl_settings_page' ]
		);
	}

	/**
	 * Render settings page.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_settings_page() { ?>
        <div class="wrap">
            <h1><?php _e( 'DocCheck Login', 'doccheck-login' ); ?></h1>
            <form method="post" action="options.php"><?php

				// Display settings
				do_settings_sections( $this->plugin_name );
				settings_fields( 'dcl-settings' );
				submit_button();

				?></form>
        </div>
	<?php }

	/**
	 * Help box on settings page.
	 */
	public function dcl_settings_page_help() {
		$screen = get_current_screen();

		// Help tab: first steps
		$screen->add_help_tab( [
			'id'      => 'first_steps',
			'title'   => __( 'First steps', 'doccheck-login' ),
			'content' =>
				'<p>' . __( 'In order for this plugin to work you\'ll need:', 'doccheck-login' ) . '</p>'
				. '<ol>'
				. '<li>' . sprintf( __( 'A DocCheck login (see <a href="%s" target="_blank">DocCheck Flexikon</a> for help).', 'doccheck-login' ), $this->dcl_support_login_link ) . '</li>'
				. '<li>' . sprintf( __( 'To request a client secret, please send an email to <a href="mailto:%1$s">%1$s</a> referencing your login ID.', 'doccheck-login' ), $this->dcl_support_client_secret_email ) . '<br/>'
				. __( 'Note that we recommend exchange of the secret via a secure communications channel, so make sure to include your phone number if desired.', 'doccheck-login' )
				. '</li>'
				. '<li>' . sprintf( __( 'Fill in the form below according to the information given by the DocCheck login administration backend (see <a href="%1$s" target="_blank">%2$s</a>).', 'doccheck-login' ), $this->dcl_support_cream_link, 'CReaM' ) . '<br/>'
				. '</ol>'
		] );

		// Help tab: shortcodes
		$screen->add_help_tab( [
			'id'      => 'shortcodes',
			'title'   => __( 'Shortcodes', 'doccheck-login' ),
			'content' =>
				'<p>' . __( 'There are a few shortcods available:', 'doccheck-login' ) . '</p>'

				. '<p><strong>' . __( 'Login form', 'doccheck-login' ) . '</strong></p>'
				. '<p><code>[dc-login]</code></p>'

				. '<p><strong>' . __( 'Logout link', 'doccheck-login' ) . '</strong></p>'
				. '<p><code>[dc-logout-link]</code></p>'

				. '<p><strong>' . __( 'HTML sitemap', 'doccheck-login' ) . '</strong></p>'
				. '<p><code>[dc-html-sitemap]</code></p>'
				. '<p>' . __( 'Displays sitemap with or without access restricted pages, depending on the users login state.', 'doccheck-login' ) . '</p>'

				. '<p><strong>' . __( 'Hide content', 'doccheck-login' ) . '</strong></p>'
				. '<p><code>[dc-hide-content]Hidden content[/dc-hide-content]</code></p>'
				. '<p>' . __( 'Hides content for users that are not logged in via DocCheck.', 'doccheck-login' ) . '</p>'
		] );
	}

	/**
	 * Register settings.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_register_settings() {
		$settings_page            = $this->plugin_name;
		$settings_section_general = $this->plugin_name . '-general-settings';
		$settings_section_form    = $this->plugin_name . '-form-settings';

		// Sections
		add_settings_section(
			$settings_section_general,
			__( 'General', 'doccheck-login' ),
			[ $this, 'dcl_section_general_callback' ],
			$settings_page
		);

		add_settings_section(
			$settings_section_form,
			__( 'Form presets', 'doccheck-login' ),
			[ $this, 'dcl_section_form_callback' ],
			$settings_page
		);

		// Fields
		$settings = [
			// Section general
			[
				'name'    => 'dcl_general_login_id',
				'title'   => __( 'Login id', 'doccheck-login' ),
				'section' => $settings_section_general,
				'type'    => 'text',
			],
			[
				'name'    => 'dcl_general_login_id_debug',
				'title'   => __( 'Login id (debug mode)', 'doccheck-login' ),
				'section' => $settings_section_general,
				'type'    => 'text'
			],
			[
				'name'    => 'dcl_client_secret',
				'title'   => __( 'Client secret', 'doccheck-login' ),
				'section' => $settings_section_general,
				'type'    => 'text',
			],
			[
				'name'    => 'dcl_client_secret_debug',
				'title'   => __( 'Client secret (debug mode)', 'doccheck-login' ),
				'section' => $settings_section_general,
				'type'    => 'text',
			],
			[
				'name'    => 'dcl_general_login_page_id',
				'title'   => __( 'Login page', 'doccheck-login' ),
				'section' => $settings_section_general,
				'type'    => 'select',
				'atts'    => [
					'select_options' => $this->dcl_get_pages_for_select(),
					'default'        => $this->dcl_page_login_id,
				]
			],
			[
				'name'    => 'dcl_general_professional_circles_page_id',
				'title'   => __( 'Target page', 'doccheck-login' ),
				'section' => $settings_section_general,
				'type'    => 'select',
				'atts'    => [
					'select_options' => $this->dcl_get_pages_for_select(),
					'default'        => $this->dcl_page_professional_circles_id,
				]
			],
			[
				'name'    => 'dcl_general_hide_menu_items',
				'title'   => __( 'Menu items', 'doccheck-login' ),
				'section' => $settings_section_general,
				'type'    => 'checkbox',
				'atts'    => [
					'label' => __( 'Hide access restricted page from menus', 'doccheck-login' ),
				]
			],
			// Section form
			[
				'name'    => 'dcl_form_size',
				'title'   => __( 'Size', 'doccheck-login' ),
				'section' => $settings_section_form,
				'type'    => 'select',
				'atts'    => [
					'select_options' => $this->dcl_get_form_sizes()
				]
			],
			[
				'name'    => 'dcl_form_language',
				'title'   => __( 'Language', 'doccheck-login' ),
				'section' => $settings_section_form,
				'type'    => 'select',
				'atts'    => [
					'select_options' => $this->dcl_get_form_languages()
				]
			],
			[
				'name'    => 'dcl_general_set_language_by_wpml',
				'title'   => __( 'Sprache (WPML)', 'doccheck-login' ),
				'section' => $settings_section_form,
				'type'    => 'checkbox',
				'atts'    => [
					'label' => __( 'Set language automatically using WPML', 'doccheck-login' ),
				]
			],
			[
				'name'    => 'dcl_form_custom_template',
				'title'   => __( 'Custom template', 'doccheck-login' ),
				'section' => $settings_section_form,
				'type'    => 'text',
			],
			[
				'name'    => 'dcl_form_custom_template_width',
				'title'   => __( 'Custom template width', 'doccheck-login' ),
				'section' => $settings_section_form,
				'type'    => 'text',
			],
			[
				'name'    => 'dcl_form_custom_template_height',
				'title'   => __( 'Custom template height', 'doccheck-login' ),
				'section' => $settings_section_form,
				'type'    => 'text',
			],
		];

		// Add and register settings
		foreach ( $settings as $setting ) {
			$setting['atts']['label_for']   = $setting['name'];
			$setting['atts']['option_name'] = $setting['name'];

			add_settings_field(
				$setting['name'],
				$setting['title'],
				[ $this, 'dcl_render_' . $setting['type'] . '_field' ],
				$settings_page,
				$setting['section'],
				$setting['atts']
			);

			$sanitze_callback = isset( $setting['sanitize_callback'] ) ? $setting['sanitize_callback'] : '';

			register_setting( 'dcl-settings', $setting['name'], $sanitze_callback );
		}
	}

	/**
	 * Display settings section general.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_section_general_callback() {
		// We currently have no intro text for this section.
	}

	/**
	 * Display settings section form.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_section_form_callback() {
		// We currently have no intro text for this section.
	}

	/**
	 * Render text field.
	 *
	 * @param   $atts
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_render_text_field( $atts ) {
		// Set attributes
		$option_name = $atts['option_name'];
		$placeholder = isset( $atts['placeholder'] ) ? $atts['placeholder'] : '';
		$default     = isset( $atts['default'] ) ? $atts['default'] : false;
		$description = isset( $atts['description'] ) ? $atts['description'] : false;

		// Get option value
		$value = get_option( $option_name, $default );

		// Field markup
		?><input type="text" class="regular-text"
                 id="<?php echo $option_name; ?>"
                 name="<?php echo $option_name; ?>"
                 value="<?php echo esc_attr( $value ) ?>"
                 placeholder="<?php echo esc_attr( $placeholder ) ?>" />

		<?php if ( $description ) : ?>
            <p class="description"><?php echo $description; ?></p>
		<?php endif; ?>
	<?php }

	/**
	 * Render select field.
	 *
	 * @param   $atts
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_render_select_field( $atts ) {
		// Set attributes
		$option_name    = $atts['option_name'];
		$select_options = isset( $atts['select_options'] ) ? $atts['select_options'] : [];
		$default        = isset( $atts['default'] ) ? $atts['default'] : false;
		$description    = isset( $atts['description'] ) ? $atts['description'] : false;

		// Get option value
		$value = get_option( $option_name, $default ); ?>

        <select name="<?php echo $option_name ?>" id="<?php echo $option_name ?>">
			<?php foreach ( $select_options as $option_key => $option_value ) : ?>
                <option value="<?php echo $option_key ?>" <?php selected( $value, $option_key ); ?>><?php
					echo $option_value
					?></option>
			<?php endforeach; ?>
        </select>

		<?php if ( $description ) : ?>
            <p class="description"><?php echo $description; ?></p>
		<?php endif; ?>
	<?php }

	/**
	 * Render checkbox.
	 *
	 * @param   $atts
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_render_checkbox_field( $atts ) {
		// Set attributes
		$option_name = $atts['option_name'];
		$label       = isset( $atts['label'] ) ? $atts['label'] : '';
		$default     = isset( $atts['default'] ) ? $atts['default'] : 0;
		$description = isset( $atts['description'] ) ? $atts['description'] : false;

		// Get option value
		$value = get_option( $option_name, $default );

		// Field markup
		?><label for="<?php echo $option_name; ?>">
        <input type="checkbox"
               id="<?php echo $option_name; ?>"
               name="<?php echo $option_name; ?>"
               value="1"
			<?php checked( $value, 1, true ); ?> />
		<?php echo $label; ?>
        </label>

		<?php if ( $description ) : ?>
            <p class="description"><?php echo $description; ?></p>
		<?php endif; ?>
	<?php }


	/**
	 * Get pages for select.
	 *
	 * @return  array $options With available pages.
	 * @since   1.0.0
	 * @access  public
	 */
	private function dcl_get_pages_for_select() {
		$options = [];
		$pages   = get_pages();

		// Add front page as default
		$options[0] = __( 'Front Page', 'doccheck-login' );

		foreach ( $pages as $page ) {
			if ( $page->post_parent > 0 ) {
				$options[ $page->ID ] = '- ' . $page->post_title;
				continue;
			}

			$options[ $page->ID ] = $page->post_title;
		}

		return $options;
	}
}