<?php

namespace DCL\Client;

use DCL\Includes\DCL_Base;

/**
 * Client: basic functionality.
 *
 * Defines public facing, basic plugin functionality.
 *
 * @package    DCL\Client
 * @author     antwerpes ag <opensource@antwerpes.com>
 */
class DCL_Client extends DCL_Base {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of this plugin.
	 */
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param   string $plugin_name The name of the plugin.
	 * @param   string $version The version of this plugin.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function __construct( $plugin_name, $version ) {
		parent::__construct();

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Filter search results.
	 *
	 * If the user is not logged in he should not get
	 * access restricted posts or pages via the search.
	 *
	 * @param   $query
	 *
	 * @return  mixed
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_filter_search_results( $query ) {
		if ( (bool) $query->is_search && false === $this->dcl_has_logged_in_user() ) {
			$meta_query = [
				[
					'restrict_access_exists' => [
						'key'     => 'dcl_restrict_access',
						'compare' => 'NOT EXISTS',
					]
				]
			];

			$query->set( 'meta_query', $meta_query );

			if ( ! isset( $query->query_vars['post_type'] ) ) {
				$query->set( 'post_type', [ 'page', 'post' ] );
			}
		}

		return $query;
	}

	/**
	 * Filter menu items.
	 *
	 * If user checked the option, this function will filter
	 * out all access restricted pages from menus.
	 *
	 * @param $items
	 *
	 * @return mixed
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_filter_menus( $items ) {
		$filter_menus = (bool) get_option( 'dcl_general_hide_menu_items' );

		if ( $filter_menus && false === $this->dcl_has_logged_in_user() ) {
			foreach ( $items as $key => $item ) {
				$access_restriction = get_post_meta( $item->object_id, 'dcl_restrict_access', true );

				if ( $access_restriction === 'on' ) {
					unset( $items[ $key ] );
				}
			}
		}

		return $items;
	}

	/**
	 * Access redirect.
	 *
	 * Redirects user to login page if the access is restricted
	 * and there is no cookie to validate that the user is a DocCheck user.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_access_redirect() {

		// Skip ahead if page is not access restricted
		// or the client is a DocCheck crawler
		// or the user is "logged in" via DocCheck
		if ( false === $this->dcl_is_access_restricted()
		     || $this->dcl_is_client_dc_crawler()
		) {
			return;
		}

		// If the user is "logged in" but a login is also in progress
		// Redirect if necessary
		if ( $this->dcl_has_logged_in_user() ) {
			if ( $this->dcl_login_in_progress() && $this->dcl_is_valid_login_time() ) {
				$this->dcl_do_login();
			} else {
				return;
			}
		}

		// Do login if requested
		if ( $this->dcl_login_in_progress() ) {
			$this->dcl_do_login();
		}

		// Seems like nothing quite fits,
		// this page should be protected by the DocCheck login
		$this->dcl_redirect_to_login();
	}

	/**
	 * "Login" DocCheck user.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_do_login() {

		// Check if this request still makes sense
		if ( $this->dcl_is_valid_login_time() ) {

			// Get login id and client secret
			if ( WP_DEBUG ) {
				$login_id      = get_option( 'dcl_general_login_id_debug' );
				$client_secret = get_option( 'dcl_client_secret_debug' );
			} else {
				$login_id      = get_option( 'dcl_general_login_id' );
				$client_secret = get_option( 'dcl_client_secret' );
			}

			// Get login code from request
			$login_code = isset( $_GET['code'] ) ? sanitize_text_field( $_GET['code'] ) : null;

			// Send POST request to DocCheck oauth service
			$response = $this->dcl_request_access_token(
				$login_id,
				$client_secret,
				$login_code
			);

			// Success
			// Received access token
			if ( property_exists( $response, 'access_token' ) ) {
				// Save token to session
				$this->dcl_set_session_token( $response->access_token );

				// Remove parameters from URL
				wp_redirect( get_permalink() );
				exit;
			}
		}

		// Error
		// Write to error log and redirect to login page
		if ( isset( $response ) && property_exists( $response, 'error' ) ) {
			error_log( 'DocCheck access token request error: ' . $response->error );
		}

		$this->dcl_redirect_to_login();
		exit();
	}

	/**
	 * "Logout" DocCheck user.
	 *
	 * Kills the session so the user counts as logged out.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function dcl_do_logout() {
		$redirect_url = home_url();

		// If the nonce is okay, remove the logged in cookie
		if ( wp_verify_nonce( $_REQUEST['dcl_logout_nonce'], 'dcl_logout_nonce' ) ) {
			session_destroy();
		}

		wp_redirect( $redirect_url );
		exit();
	}

	/**
	 * Check if there is a login request
	 * that should be processed.
	 *
	 * @return bool
	 * @since   1.0.0
	 */
	private function dcl_login_in_progress() {
		if ( isset( $_GET['dc_timestamp'] ) && isset( $_GET['code'] ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Check login request time.
	 *
	 * Makes sure the time when the request was sent and the current time are not too far apart.
	 * So you can't send a login request repeatedly.
	 *
	 * @return bool
	 * @since   1.0.0
	 */
	private function dcl_is_valid_login_time() {
		$login_timestamp = isset( $_GET['dc_timestamp'] ) ? $_GET['dc_timestamp'] : null;
		$refresh_time    = time() - ( 60 * 5 );

		return $login_timestamp >= $refresh_time;
	}

	/**
	 * Request access token from oauth service.
	 *
	 * @param $login_id
	 * @param $client_secret
	 * @param $login_code
	 *
	 * @return array|mixed|object
	 * @since   1.0.0
	 */
	private function dcl_request_access_token( $login_id, $client_secret, $login_code ) {
		$curl = curl_init( self::DCL_OAUTH_SERVICE );
		curl_setopt( $curl, CURLOPT_POST, true );
		curl_setopt( $curl, CURLOPT_POSTFIELDS, [
			'client_id'     => $login_id,
			'client_secret' => $client_secret,
			'code'          => $login_code,
			'grant_type'    => 'authorization_code'
		] );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, 1 );
		$auth = curl_exec( $curl );

		return json_decode( $auth );
	}

	/**
	 * Request access token validation
	 * from oAuth API.
	 *
	 * @param $saved_access_token
	 *
	 * @return mixed
	 */
	private function dcl_request_token_validation( $saved_access_token ) {
		$curl = curl_init( self::DCL_CHECK_TOKEN . '?access_token=' . $saved_access_token );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, 1 );

		$auth     = curl_exec( $curl );
		$response = json_decode( $auth );

		return $response->boolIsValid;
	}

	/**
	 * Redirect to login page.
	 *
	 * @since   1.0.0
	 */
	private function dcl_redirect_to_login() {
		$redirect_url = get_permalink();
		$login_url    = $this->dcl_get_login_permalink( $redirect_url );

		// Set redirect cookie
		$this->dcl_set_session_redirect_url( $redirect_url );

		wp_redirect( $login_url );
	}
}
